import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import '../features/call/call_page.dart';
import '../features/call/incoming_call_page.dart';
import '../services/contact_service.dart'; // Import your ContactService

class CallService {
  static const MethodChannel _channel = MethodChannel('call_service');
  static String? currentPhoneNumber;
  static String? currentDisplayName; // Store display name
  static Uint8List? currentThumbnail; // Store thumbnail
  static bool _isCallPageVisible = false;
  final ContactService _contactService = ContactService(); // Instantiate ContactService

  static final GlobalKey<NavigatorState> navigatorKey = GlobalKey<NavigatorState>();

  CallService() {
    _channel.setMethodCallHandler((call) async {
      final context = navigatorKey.currentContext;
      print('CallService: Received method ${call.method} with args ${call.arguments}');
      if (context == null) {
        print('CallService: Navigator context is null, cannot navigate');
        return;
      }

      switch (call.method) {
        case "callAdded":
          final phoneNumber = call.arguments["callId"] as String;
          final state = call.arguments["state"] as String;
          currentPhoneNumber = phoneNumber.replaceFirst('tel:', '');
          // Fetch contact info using ContactService if not already set
          await _fetchContactInfo(currentPhoneNumber!);
          print('CallService: Call added, number: $currentPhoneNumber, state: $state');
          if (state == "ringing") {
            _navigateToIncomingCallPage(context);
          } else {
            _navigateToCallPage(context);
          }
          break;
        case "callStateChanged":
          final state = call.arguments["state"] as String;
          print('CallService: State changed to $state');
          if (state == "disconnected" || state == "disconnecting") {
            _closeCallPage(context);
          } else if (state == "active" || state == "dialing") {
            _navigateToCallPage(context);
          } else if (state == "ringing") {
            _navigateToIncomingCallPage(context);
          }
          break;
        case "callEnded":
        case "callRemoved":
          print('CallService: Call ended/removed');
          _closeCallPage(context);
          currentPhoneNumber = null;
          currentDisplayName = null;
          currentThumbnail = null;
          break;
      }
    });
  }

  Future<void> _fetchContactInfo(String phoneNumber) async {
    if (currentDisplayName != null && currentThumbnail != null) return; // Already set
    try {
      final contacts = await _contactService.fetchContacts(); // Use ContactService
      for (var contact in contacts) {
        for (var phone in contact.phones) {
          if (_normalizePhoneNumber(phone.number) == _normalizePhoneNumber(phoneNumber)) {
            currentDisplayName = contact.displayName;
            currentThumbnail = contact.thumbnail;
            return;
          }
        }
      }
      // If no match found, use phone number as fallback
      currentDisplayName ??= phoneNumber;
      currentThumbnail ??= null;
    } catch (e) {
      print('CallService: Error fetching contact info: $e');
      currentDisplayName = phoneNumber;
      currentThumbnail = null;
    }
  }

  String _normalizePhoneNumber(String number) {
    return number.replaceAll(RegExp(r'[\s\-\(\)]'), '');
  }

  void _navigateToCallPage(BuildContext context) {
    if (_isCallPageVisible && ModalRoute.of(context)?.settings.name == '/call') {
      print('CallService: CallPage already visible, skipping navigation');
      return;
    }
    print('CallService: Navigating to CallPage');
    Navigator.pushReplacement(
      context,
      MaterialPageRoute(
        settings: const RouteSettings(name: '/call'),
        builder: (context) => CallPage(
          displayName: currentDisplayName ?? currentPhoneNumber!,
          phoneNumber: currentPhoneNumber!,
          thumbnail: currentThumbnail,
        ),
      ),
    ).then((_) {
      _isCallPageVisible = false;
    });
    _isCallPageVisible = true;
  }

  void _navigateToIncomingCallPage(BuildContext context) {
    if (_isCallPageVisible && ModalRoute.of(context)?.settings.name == '/incoming_call') {
      print('CallService: IncomingCallPage already visible, skipping navigation');
      return;
    }
    print('CallService: Navigating to IncomingCallPage');
    Navigator.push(
      context,
      MaterialPageRoute(
        settings: const RouteSettings(name: '/incoming_call'),
        builder: (context) => IncomingCallPage(
          displayName: currentDisplayName ?? currentPhoneNumber!,
          phoneNumber: currentPhoneNumber!,
          thumbnail: currentThumbnail,
        ),
      ),
    ).then((_) {
      _isCallPageVisible = false;
    });
    _isCallPageVisible = true;
  }

  void _closeCallPage(BuildContext context) {
    if (!_isCallPageVisible) {
      print('CallService: CallPage not visible, skipping pop');
      return;
    }
    if (Navigator.canPop(context)) {
      print('CallService: Popping CallPage');
      Navigator.pop(context);
      _isCallPageVisible = false;
    }
  }

  Future<void> makeGsmCall(
    BuildContext context, {
    required String phoneNumber,
    String? displayName,
    Uint8List? thumbnail,
  }) async {
    try {
      currentPhoneNumber = phoneNumber;
      currentDisplayName = displayName ?? phoneNumber; // Use provided or fetch later
      currentThumbnail = thumbnail; // Use provided or fetch later
      print('CallService: Making GSM call to $phoneNumber');
      final result = await _channel.invokeMethod('makeGsmCall', {"phoneNumber": phoneNumber});
      print('CallService: makeGsmCall result: $result');
      if (result["status"] != "calling") {
        ScaffoldMessenger.of(context).showSnackBar(
          SnackBar(content: Text("Failed to initiate call")),
        );
        return;
      }
      // Fetch contact info if not provided
      await _fetchContactInfo(phoneNumber);
      _navigateToCallPage(context); // Navigate immediately after call initiation
    } catch (e) {
      print("CallService: Error making call: $e");
      ScaffoldMessenger.of(context).showSnackBar(
        SnackBar(content: Text("Error making call: $e")),
      );
      rethrow;
    }
  }

  Future<void> hangUpCall(BuildContext context) async {
    try {
      print('CallService: Hanging up call');
      final result = await _channel.invokeMethod('hangUpCall');
      print('CallService: hangUpCall result: $result');
      if (result["status"] != "ended") {
        ScaffoldMessenger.of(context).showSnackBar(
          SnackBar(content: Text("Failed to end call")),
        );
      }
    } catch (e) {
      print("CallService: Error hanging up call: $e");
      ScaffoldMessenger.of(context).showSnackBar(
        SnackBar(content: Text("Error hanging up call: $e")),
      );
      rethrow;
    }
  }
}